%Clear workspace
clear all
close all
clc

%Load wave data
load LensvikAndBrekstad

%Plot time series of significant wave height at Brekstad:
figure;
plot(Brekstad.Significant_wave_height);
xlabel('Time[h]')
ylabel('Hs[m]')


%% Set interval and step-length of operational criterion and reference period to be analyzed:
%Operational criterin (Hs[m]):
OPwf=[0.5:0.25:1.5];
%Reference period [h]
Tr=[0:0.1:15]; 

%% Cumulative distribution waveheight (WEIBUL)
%Choose wave data to analyze:
wave=Brekstad.Significant_wave_height; 

%Find all measurements with Hs=0m and change to Hs=0.01m:
%(wblfit requires only positive non-zero values.
%Will not affect results as all analyzed operational criterions are > than
%0.01m)

m=length(wave);
for a=1:m
    if wave(a)==0
        wave(a)=0.01;
    end
end

%Estimate weibull parameters for waveheights:
param=wblfit(wave);

x=(0:0.001:1.5);
%Probability density function f:
f1=param(2)/param(1)*(x/param(1)).^(param(2)-1).*exp(-(x/param(1)).^param(2)); %https://se.mathworks.com/help/stats/wblfit.html 

%Plot fitted probability density function together with histogram of forecasted significant wave
%heights:
figure;
plot(x,f1)
xlabel('Hs')
ylabel('f(Hs')
hold on
histogram(wave,'Normalization','pdf');
title 'BREKSTAD - Weibull PDF fitted to normalized histogram'

%Cumulative distribution function F:
F1=1-exp(-(x/param(1)).^param(2));

%Plot fitted cumulative probability distribution function together with empirical 
%cumulative probability distribution of forecasted significant wave:
figure;
c=cdfplot(wave);
c.Color='k';
xlabel('Hs[m]');
ylabel('F(Hs)');

grid off
hold on

plot(x,F1,'k','LineWidth',1)
title 'Empirical vs theoretical CDF'

hold off


%% Calcuate Probability that Hs is less than the evaluated OPwfs:
for i=1:length(OPwf)
    F_val(i)=wblcdf(OPwf(i),param(1),param(2));
end

%% Length of calms
figure;
plot(wave,'k')
xlabel('time[h]')
ylabel('Hs[m]')
color('blk')

for u=1:length(OPwf) %Calculate for all operational criterions
    i=1; %Hour in calm
    j=1; %Hour number
    k=1; % Calm number

for j = 1:length(wave) %Check wave height for all elements in the time series
    if wave(j)<=OPwf(u) %Check if wave height at the time is lower than operational criterion
        CalmVector(i)=j; %If yes, the element is part of a calm
        i=i+1; 
        Lengths(k)=length(CalmVector); %The length of the current calm is updated
    else
        CalmVector=0; %If no, the calm is completed
        k=k+1;        %Checking next calm
        i=1;          %Start on first hour in the calm
    end
end

%Lengths of all calms from current operational criterion:
LengthOfCalms_u=nonzeros(Lengths); 

Lengths=0 % RESET for next loop (next operational criterion)

%Estimate weibull parameters for length of calms for current operational
%criterion:
param_calm=wblfit(LengthOfCalms_u);

x_calm=(0:0.10:1000);

%Probability density function f:
f1_calm=param_calm(2)/param_calm(1)*(x_calm/param_calm(1)).^(param_calm(2)-1).*exp(-(x_calm/param_calm(1)).^param_calm(2)); 

%Cumulative distribution function F:
F1_calm=1-exp(-(x_calm/param_calm(1)).^param_calm(2));

%Plot empirical cumulative distribution function vs estimated weibull
%function:
figure;
cc=cdfplot(LengthOfCalms_u);
cc.Color='k';

hold on
plot(x_calm,F1_calm,'k','LineWidth',1)
title(['Hs=' num2str(OPwf(u))])
xlabel('Length of calm[h]');
ylabel('F(Length of calm)');
grid off
axis([0 1000 0 1])


%% Calculate availability of operation:
for v=1:length(Tr)
    
    probCalm(u,v)=1-wblcdf(Tr(v),param_calm(1),param_calm(2)); %[wave height,referance period]
    
    ProbWork(u,v)=F_val(u)*probCalm(u,v);
   
end
    
end

% Plot availability of the operation as function of referance period for
% each evaluated operational criterion:
%(2016aplot)
figure;
for i=1:length(OPwf)
    plot(Tr,100*ProbWork(i,:),'linewidth',1)
    xlabel('Reference period [h]')
    ylabel('Availability of operation [%]')
    axis([min(Tr) max(Tr) 0 100])
    set(gca,'xtick',[0:1:max(Tr)]);
    set(gca,'ytick',[0:10:100]);
    hold on
end
legend('OpLim=0.5m', 'OPLim=0.75m', 'OPLim=1m', 'OPLim=1.25m', 'OPLim=1.5m')

%PLOTTING WITH MARKERS ON ONLY SOME OF THE DATA POINTS. NB! THIS PLOT IS
%ONLY APPLICABLE FOR MATLAB R2016b OR NEWER:
figure;
for i=1:length(OPwf)
    marker=['^','o','X','+','d'];
    p=plot(Tr,100*ProbWork(i,:));
    p.Color='k';
    p.LineWidth=1;
    p.Marker=marker(i);
    p.MarkerIndices=[40:40:length(Tr)-5];
    xlabel('Duration of operation [h]') % Reference period
    ylabel('Availability of operation [%]')
    axis([min(Tr) max(Tr) 0 100])
    set(gca,'xtick',[0:1:max(Tr)]);
    set(gca,'ytick',[0:10:100]);
    hold on
end
legend('OpLim=0.5m', 'OPLim=0.75m', 'OPLim=1m', 'OPLim=1.25m', 'OPLim=1.5m')
